<?php
	if ( !defined ( 'DIR_CORE' )) {
		header ( 'Location: static_pages/' );
	}

class ExtensionRewardPoints extends Extension {

	public $errors = array();
	public $data = array();
	protected $registry;

	public function  __construct() {
		$this->registry = Registry::getInstance();
	}

	public function onControllerPagesCatalogProduct_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogProductImages_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogProductRelations_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogProductOptions_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogProductPromotions_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogProductExtensions_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogProductLayout_InitData() {
		$this->_get_products_tabs();
	}

	public function onControllerPagesCatalogCategory_InitData() {
		$this->_get_categories_tabs();
	}

	public function onControllerPagesCatalogManufacturer_InitData() {
		$this->_get_manufacturers_tabs();
	}

	public function onControllerPagesCatalogManufacturerLayout_InitData() {
		$this->_get_manufacturers_tabs();
	}

	public function onControllerPagesSaleCustomer_InitData() {
		if ( !isset($this->baseObject->request->get['customer_id']) ) {
			return;
		}
		$this->baseObject->loadLanguage('reward_points/reward_points');
		$tabs = array(
			array(
				'link' => $this->baseObject->html->getSecureURL('sale/customer/update', '&customer_id=' . $this->baseObject->request->get['customer_id']),
				'text' => $this->baseObject->language->get('text_general'),
				'class' => 'active'
			),
			array(
				'link' => $this->baseObject->html->getSecureURL('extension/reward_points/customer', '&customer_id=' . $this->baseObject->request->get['customer_id']),
				'text' => $this->baseObject->language->get('reward_points_name'),
				'class' => ''
			)
		);

		$view = new AView($this->registry, 0);
		$view->assign('tabs',$tabs);
		$this->baseObject->view->addHookVar('extension_tabs', $view->fetch('pages/reward_points/customer_tabs.tpl'));

	}

	public function onControllerPagesCheckoutPayment_InitData() {

		$data = array();

		$this->baseObject->loadLanguage('reward_points/reward_points');
		$this->baseObject->loadModel('extension/reward_points');

		$customer_id = $this->baseObject->customer->getId();
		$points_info = $this->baseObject->model_extension_reward_points->getCustomerRewardPoints($customer_id);

		$data['fields'] = array(
			array(
				'name' => $this->baseObject->language->get('text_your_total_points'),
				'html' => isset($points_info['points_amount']) ? $points_info['points_amount'] : 0
			),
			array(
				'name' => $this->baseObject->language->get('text_order_total_points'),
				'html' => $this->baseObject->session->data['reward_points_total_cost']
			),
			array(
				'name' => $this->baseObject->language->get('text_points_to_use'),
				'html' => $this->baseObject->html->buildInput(
					array(
						'name' => 'reward_points_used',
						'value' => '',
					)
				)
			)
		);

		$data['text_reward_points'] = $this->baseObject->language->get('text_reward_points');
		$data['text_reward_points_used'] = $this->baseObject->language->get('text_reward_points_used');
		$data['apply_points'] = $this->baseObject->html->getSecureURL('extension/reward_points/apply_points');
		$data['reset_points'] = $this->baseObject->html->getSecureURL('extension/reward_points/reset_points');

		$view = new AView($this->registry, 0);
		$view->batchAssign($data);
		$this->baseObject->view->addHookVar('payment_extensions_hook', $view->fetch('pages/extension/reward_points_payment.tpl'));
	}

	public function onControllerPagesCheckoutCart_UpdateData() {

		$this->baseObject->loadLanguage('reward_points/reward_points');
		$this->baseObject->loadModel('extension/reward_points');

		$price = $this->baseObject->cart->getSubTotal();
		$points_cost = round ( $price * $this->baseObject->config->get('reward_points_per') );

		if ( $points_cost > 0 ) {
			$this->baseObject->data['totals'][] = array(
				'id' => 'reward_points',
				'title' => $this->baseObject->language->get('text_reward_points_cost') .':',
				'text' => $points_cost . ' ' . $this->baseObject->language->get('text_points'),
				'value' => $points_cost,
				'sort_order' => 1000,
				'total_type' => 'reward_points'
			);
			$this->baseObject->session->data['reward_points_total_cost'] = $points_cost;
			$this->baseObject->view->assign('totals', $this->baseObject->data['totals']);
		}


	}

	public function onControllerBlocksOrderSummary_UpdateData() {

		$this->baseObject->loadLanguage('reward_points/reward_points');
		$this->baseObject->loadModel('extension/reward_points');

		$price = $this->baseObject->cart->getSubTotal();
		$points_cost = round ( $price * $this->baseObject->config->get('reward_points_per') );

		if ( $points_cost > 0 ) {
			$this->baseObject->data['totals'][] = array(
				'id' => 'reward_points',
				'title' => $this->baseObject->language->get('text_reward_points_cost') .':',
				'text' => $points_cost . ' ' . $this->baseObject->language->get('text_points'),
				'value' => $points_cost,
				'sort_order' => 1000,
				'total_type' => 'reward_points'
			);
			$this->baseObject->session->data['reward_points_total_cost'] = $points_cost;
			$this->baseObject->view->assign('totals', $this->baseObject->data['totals']);
		}
	}

	public function onControllerPagesCheckoutConfirm_UpdateData() {

		if ( isset($this->baseObject->session->data['reward_points_error']) ) {
			$this->baseObject->view->assign('error_warning', $this->baseObject->session->data['reward_points_error']);
			unset($this->baseObject->session->data['reward_points_error']);
		}
	}

	public function onControllerPagesCheckoutSuccess_InitData() {
		if ( isset($this->baseObject->session->data['order_id']) && $this->baseObject->customer->isLogged() ) {
			$this->baseObject->loadLanguage('reward_points/reward_points');

			$this->baseObject->loadModel('extension/reward_points');
			$customer_id = $this->baseObject->customer->getId();
			$customer_points_info = $this->baseObject->model_extension_reward_points->getCustomerRewardPoints($customer_id);

			$this->baseObject->loadModel('checkout/order');
			$order = $this->baseObject->model_checkout_order->getOrder($this->baseObject->session->data['order_id']);


			if ( has_value($this->baseObject->session->data['reward_points']) ) { // Pay with reward points

				if ( $order['order_status_id'] == $this->baseObject->config->get('reward_points_order_status') ) {

					$total_points = $customer_points_info['points_amount'] - $this->baseObject->session->data['reward_points']['points_used'];
					$this->baseObject->model_extension_reward_points->setCustomerRewardPoints($customer_id, $total_points);
					$this->baseObject->model_extension_reward_points->updateHistory(
						array(
							'customer_id' => $customer_id,
							'amount' => $this->baseObject->session->data['reward_points']['points_used'],
							'action' => 'D', // decreased
							'source' => 'order',
							'source_id' => $this->baseObject->session->data['order_id'],
							'comment' => 'Order ID: ' . $this->baseObject->session->data['order_id']
						)
					);

				}

				$this->baseObject->model_extension_reward_points->setRewardPointsOrder(
					array(
						'order_id' => $this->baseObject->session->data['order_id'],
						'amount' => $this->baseObject->session->data['reward_points']['points_used'],
						'action' => 'D'
					)
				);

				unset($this->baseObject->session->data['reward_points']);
			} else {
				// Gain points for buying products

				$products = $this->baseObject->cart->getProducts();
				$points_amount = 0;
				foreach ( $products as $product ) {
					$points_amount += $this->_get_points_total($product['total'], $product['product_id']);
				}

				// If we gain some points
				if ( $points_amount > 0 ) {

					if ( $order['order_status_id'] == $this->baseObject->config->get('reward_points_order_status') ) {
						if ( !empty($customer_points_info) ) {
							$total_points = $points_amount + $customer_points_info['points_amount'];
						} else {
							$total_points = $points_amount;
						}
						$this->baseObject->model_extension_reward_points->setCustomerRewardPoints($customer_id, $total_points);
						$this->baseObject->model_extension_reward_points->updateHistory(
							array(
								'customer_id' => $customer_id,
								'amount' => $points_amount,
								'action' => 'I', // increased
								'source' => 'order',
								'source_id' => $this->baseObject->session->data['order_id'],
								'comment' => 'Order ID: ' . $this->baseObject->session->data['order_id']
							)
						);
					}

					$this->baseObject->model_extension_reward_points->setRewardPointsOrder(
						array(
							'order_id' => $this->baseObject->session->data['order_id'],
							'amount' => $points_amount,
							'action' => 'I'
						)
					);
				}

			}


		}
	}

	public function onControllerPagesSaleOrder_InitData() {
		if ( func_get_arg(0) == 'history'
			&& $this->baseObject->request->server['REQUEST_METHOD'] == 'POST'
			&& $this->baseObject->user->canModify('sale/order')
			&& isset($this->baseObject->request->post['order_status_id'])
		) {
			$this->baseObject->loadLanguage('reward_points/reward_points');
			$this->baseObject->loadModel('extension/reward_points');

			$order = $this->baseObject->model_sale_order->getOrder($this->baseObject->request->get['order_id']);

			$points_order = $this->baseObject->model_extension_reward_points->getRewardPointsOrder($order['order_id']);

			if ( !empty($points_order) ) {

				$customer_points_info = $this->baseObject->model_extension_reward_points->getCustomerRewardPoints($order['customer_id']);

				if ( $order['order_status_id'] == $this->baseObject->config->get('reward_points_order_status') ) {
					if ( $this->baseObject->request->post['order_status_id'] != $order['order_status_id'] ) {
						// cancel points
						if ( $points_order['action'] == 'I' ) {
							$total_points = $customer_points_info['points_amount'] - $points_order['amount'];
							$action = 'D';
						} else {
							$total_points = $customer_points_info['points_amount'] + $points_order['amount'];
							$action = 'I';
						}

						$this->baseObject->loadModel('localisation/order_status');

						$before = $this->baseObject->model_localisation_order_status->getOrderStatus($order['order_status_id']);
						$after = $this->baseObject->model_localisation_order_status->getOrderStatus($this->baseObject->request->post['order_status_id']);

						$this->baseObject->model_extension_reward_points->setCustomerRewardPoints($order['customer_id'], $total_points);
						$this->baseObject->model_extension_reward_points->updateHistory(
							array(
								'customer_id' => $order['customer_id'],
								'amount' => $points_order['amount'],
								'action' => $action,
								'source' => 'order',
								'source_id' => $order['order_id'],
								'comment' => 'Order ID: ' . $order['order_id'] . '. ' . $before['name'] . ' -> ' . $after['name']
							)
						);

					}
				} else {
					if ( $this->baseObject->request->post['order_status_id'] == $this->baseObject->config->get('reward_points_order_status') ) {
						// apply points
						if ( $points_order['action'] == 'D' ) {
							$total_points = $customer_points_info['points_amount'] - $points_order['amount'];
						} else {
							$total_points = $customer_points_info['points_amount'] + $points_order['amount'];
						}

						$this->baseObject->loadModel('localisation/order_status');

						$before = $this->baseObject->model_localisation_order_status->getOrderStatus($order['order_status_id']);
						$after = $this->baseObject->model_localisation_order_status->getOrderStatus($this->baseObject->request->post['order_status_id']);

						$this->baseObject->model_extension_reward_points->setCustomerRewardPoints($order['customer_id'], $total_points);
						$this->baseObject->model_extension_reward_points->updateHistory(
							array(
								'customer_id' => $order['customer_id'],
								'amount' => $points_order['amount'],
								'action' => $points_order['action'],
								'source' => 'order',
								'source_id' => $order['order_id'],
								'comment' => 'Order ID: ' . $order['order_id'] . '. ' . $before['name'] . ' -> ' . $after['name']
							)
						);
					}
				}
			}
		}
	}

	public function onControllerPagesProductProduct_UpdateData() {

		if ( $this->baseObject->customer->isLogged() ) {
			if ( isset($this->baseObject->request->get['product_id']) ) {
				$product_id = $this->baseObject->request->get['product_id'];
			} else {
				$product_id = 0;
			}

			$price = isset($this->baseObject->data['special_num']) ? $this->baseObject->data['special_num'] : $this->baseObject->data['price_num'];

			$points_amount = $this->_get_points_total($price, $product_id);

			$view = new AView($this->registry, 0);

			$this->baseObject->loadLanguage('reward_points/reward_points');
			$view->assign('text_reward_points_gained', $this->baseObject->language->get('text_reward_points_gained'));
			$view->assign('text_reward_points_cost', $this->baseObject->language->get('text_reward_points_cost'));
			$view->assign('points_amount', $points_amount);

			$points_cost = round($price * $this->baseObject->config->get('reward_points_per'));
			$view->assign('points_cost', $points_cost);
			$this->baseObject->view->addHookVar('extended_product_options', $view->fetch('pages/extension/reward_points_product.tpl'));
		}
	}

	public function onControllerPagesAccountAccount_UpdateData() {
		$this->baseObject->loadLanguage('reward_points/reward_points');

		$link = '<li><a href="' . $this->baseObject->html->getSecureURL('extension/reward_points_history') . '">'
			. $this->baseObject->language->get('text_my_reward_points')
			. '</a></li>';

		$this->baseObject->view->addHookVar('account_links', $link);

	}

	public function onControllerResponsesListingGridOrder_InitData() {

		if ( $this->baseObject->user->canModify('listing_grid/order') && has_value($this->baseObject->request->post['order_status_id']) ) {

			$this->baseObject->loadModel('sale/order');

			foreach ( $this->baseObject->request->post['order_status_id'] as $order_id => $status_id ) {

				$this->baseObject->loadLanguage('reward_points/reward_points');
				$this->baseObject->loadModel('extension/reward_points');

				$order = $this->baseObject->model_sale_order->getOrder($order_id);

				$points_order = $this->baseObject->model_extension_reward_points->getRewardPointsOrder($order_id);

				if ( !empty($points_order) ) {

					$customer_points_info = $this->baseObject->model_extension_reward_points->getCustomerRewardPoints($order['customer_id']);

					if ( $order['order_status_id'] == $this->baseObject->config->get('reward_points_order_status') ) {
						if ( $status_id != $order['order_status_id'] ) {
							// cancel points
							if ( $points_order['action'] == 'I' ) {
								$total_points = $customer_points_info['points_amount'] - $points_order['amount'];
								$action = 'D';
							} else {
								$total_points = $customer_points_info['points_amount'] + $points_order['amount'];
								$action = 'I';
							}

							$this->baseObject->loadModel('localisation/order_status');

							$before = $this->baseObject->model_localisation_order_status->getOrderStatus($order['order_status_id']);
							$after = $this->baseObject->model_localisation_order_status->getOrderStatus($status_id);

							$this->baseObject->model_extension_reward_points->setCustomerRewardPoints($order['customer_id'], $total_points);
							$this->baseObject->model_extension_reward_points->updateHistory(
								array(
									'customer_id' => $order['customer_id'],
									'amount' => $points_order['amount'],
									'action' => $action,
									'source' => 'order',
									'source_id' => $order_id,
									'comment' => 'Order ID: ' . $order_id . '. ' . $before['name'] . ' -> ' . $after['name']
								)
							);

						}
					} else {
						if ( $status_id == $this->baseObject->config->get('reward_points_order_status') ) {
							// apply points
							if ( $points_order['action'] == 'D' ) {
								$total_points = $customer_points_info['points_amount'] - $points_order['amount'];
							} else {
								$total_points = $customer_points_info['points_amount'] + $points_order['amount'];
							}

							$this->baseObject->loadModel('localisation/order_status');

							$before = $this->baseObject->model_localisation_order_status->getOrderStatus($order['order_status_id']);
							$after = $this->baseObject->model_localisation_order_status->getOrderStatus($status_id);

							$this->baseObject->model_extension_reward_points->setCustomerRewardPoints($order['customer_id'], $total_points);
							$this->baseObject->model_extension_reward_points->updateHistory(
								array(
									'customer_id' => $order['customer_id'],
									'amount' => $points_order['amount'],
									'action' => $points_order['action'],
									'source' => 'order',
									'source_id' => $order_id,
									'comment' => 'Order ID: ' . $order_id . '. ' . $before['name'] . ' -> ' . $after['name']
								)
							);
						}
					}
				}

			}
		}
	}

	private function _get_points_total($price, $product_id) {
		$points_amount = 0;

		$this->baseObject->loadModel('extension/reward_points');
		$group_id = $this->baseObject->customer->getCustomerGroupId();
		$points_group = $this->baseObject->model_extension_reward_points->getRewardPoints(
			$group_id,
			'customer_group',
			$group_id
		);

		if ( !empty($points_group) ) {
			$points_amount += $this->_get_points_amount($price, $points_group['amount'], $points_group['amount_modificator']);
		}

		if ( empty($points_group) || $points_group['combine'] ) {

			if ( $manufacturer_id = $this->baseObject->model_extension_reward_points->getProductManufacturerId($product_id) ) {
				$points_manufacturer = $this->baseObject->model_extension_reward_points->getRewardPoints(
					$group_id,
					'manufacturer',
					$manufacturer_id
				);
				if ( !empty($points_manufacturer) ) {
					$points_amount += $this->_get_points_amount($price, $points_manufacturer['amount'], $points_manufacturer['amount_modificator']);
				}
			}
			$categories = $this->baseObject->model_extension_reward_points->getProductCategories($product_id);

			foreach ( $categories as $category_id ) {
				if ( $category_id ) {
					$points_category = $this->baseObject->model_extension_reward_points->getRewardPoints(
						$group_id,
						'category',
						$category_id
					);
					if ( !empty($points_category) ) {
						$points_amount += $this->_get_points_amount($price, $points_category['amount'], $points_category['amount_modificator']);
					}
				}
			}

			$points_product = $this->baseObject->model_extension_reward_points->getRewardPoints(
				$group_id,
				'product',
				$product_id
			);
			if ( !empty($points_product) ) {
				$points_amount += $this->_get_points_amount($price, $points_product['amount'], $points_product['amount_modificator']);
			}
		}
		return $points_amount;
	}

	private function _get_points_amount($price, $points_amount, $points_modificator) {
		if ( $points_modificator == 'a' ) {
			return $points_amount;
		}
		// TODO: intval or round???
		return intval($price * $points_amount / 100);
	}

	private function _get_products_tabs () {
		$this->baseObject->loadLanguage('reward_points/reward_points');

		$this->data['tabs'] = array();

		$this->data['tabs'][] = array(
			'href' => $this->baseObject->html->getSecureURL(
				'extension/reward_points',
				'&object_id=' . $this->baseObject->request->get['product_id'] .
				'&object_name=product'
			),
			'text' => $this->baseObject->language->get('reward_points_name'),
			'class' => ''
		);

		$view = new AView($this->registry, 0);
		$view->batchAssign($this->data);
		$this->baseObject->view->addHookVar('extension_tabs', $view->fetch('pages/reward_points/tabs.tpl'));
	}

	private function _get_categories_tabs () {
		$this->baseObject->loadLanguage('reward_points/reward_points');

		$this->data['tabs'] = array();

		$this->data['tabs'][] = array(
			'href' => $this->baseObject->html->getSecureURL(
				'extension/reward_points',
				'&object_id=' . $this->baseObject->request->get['category_id'] .
				'&object_name=category'
			),
			'text' => $this->baseObject->language->get('reward_points_name'),
			'class' => ''
		);

		$view = new AView($this->registry, 0);
		$view->batchAssign($this->data);
		$this->baseObject->view->addHookVar('extension_tabs', $view->fetch('pages/reward_points/tabs.tpl'));
	}

	private function _get_manufacturers_tabs () {
		$this->baseObject->loadLanguage('reward_points/reward_points');

		$this->data['tabs'] = array();

		$this->data['tabs'][] = array(
			'href' => $this->baseObject->html->getSecureURL(
				'extension/reward_points',
				'&object_id=' . $this->baseObject->request->get['manufacturer_id'] .
				'&object_name=manufacturer'
			),
			'text' => $this->baseObject->language->get('reward_points_name'),
			'class' => ''
		);

		$view = new AView($this->registry, 0);
		$view->batchAssign($this->data);
		$this->baseObject->view->addHookVar('extension_tabs', $view->fetch('pages/reward_points/tabs.tpl'));
	}

}